function score = retrievalScore(R,CQ,CR,M)
% scores = retrievalScore(R,CQ,CR)
% Computes the retrieval score for a given retrieval ranking.
% Variables:
% score - retrieval score.
% R - ranking (nQueries x nRankings).
% CQ - class of query.
% CR - class of each idx ranking.
% M - evaluation measure
%   NN - nearest neighbour.
%   FT - first tier.
%   ST - second tier.
%   DCG - dcg.
%
% David Pickup 2014

switch M
    case 'NN'
        score = nearestNeighbour(R,CQ,CR);
    case 'FT'
        score = firstTier(R,CQ,CR);
    case 'ST'
        score = secondTier(R,CQ,CR);
    case 'DCG'
        score = DCG(R,CQ,CR);
end

return;

%% Functions

function NN = nearestNeighbour(R,CQ,CR)%rankings,C)
nModels = size(R,1);
count = 0;
count2 = 0;
for i = 1:nModels
    idx = find(CR(R(i,:))==CQ(i));
    nClass = numel(idx);
    if nClass == 0
        warning(['No models in query ' int2str(i) '''s class.']);
        continue;
    end
    if CQ(i) == CR(R(i,1))
        count = count + 1;
    end
    count2 = count2 + 1;
end
NN = count / count2;
return;

function FT = firstTier(R,CQ,CR)%rankings,C)
nModels = size(R,1);
count = 0;
count2 = 0;
for i = 1:nModels
    idx = find(CR(R(i,:))==CQ(i));
    nClass = numel(idx);
    if nClass == 0
        warning(['No models in query ' int2str(i) '''s class.']);
        continue;
    end
    idx = find(CR(R(i,1:(nClass))) == CQ(i));
    count = count + (numel(idx) / nClass);
    count2 = count2 + 1;
end
FT = count / count2;
return;

function ST = secondTier(R,CQ,CR)%rankings,C)
nModels = size(R,1);
count = 0;
count2 = 0;
for i = 1:nModels
    idx = find(CR(R(i,:))==CQ(i));
    nClass = (numel(idx));
    if nClass == 0
        warning(['No models in query ' int2str(i) '''s class.']);
        continue;
    end
    idx = find(CR(R(i,1:min(2*(nClass),numel(R(i,:))))) == CQ(i));
    count = count + (numel(idx) / nClass);
    count2 = count2 + 1;
end
ST = count / count2;
return;

function DCG = DCG(R,CQ,CR)%rankings,C)
nModels = size(R,1);
count = 0;
count2 = 0;
for i = 1:nModels
    idx = find(CR(R(i,:))==CQ(i));
    nClass = numel(idx);
    if nClass == 0
        warning(['No models in query ' int2str(i) '''s class.']);
        continue;
    end
    
    ideal = 1 + sum(1./log2(2:nClass));
    
    idx = find(CR(R(i,:)) == CQ(i));
    tmp = find(idx == 1);
    if ~isempty(tmp)
        idx(tmp) = [];
        DCGi = 1 + sum(1./log2(idx));
    else
        DCGi = sum(1./log2(idx));
    end
    count = count + (DCGi / ideal);
    count2 = count2 + 1;
end
DCG = count / count2;
return;